import binascii
import os
import time
import hmac
import hashlib
import base64
import requests


class Transaction(object):
    API_KEY = ''
    API_SECRET = ''
    TOKEN = ''
    URL = ''

    def __init__(self, payload):
        self.payload = payload
        self.transaction_response = None

    def __get_transaction_response(self):
        return self.transaction_response

    def __set_transaction_response(self, response_value):
        self.transaction_response = response_value

    def __generate_hmac(self):
        # Cryptographically strong random number
        nonce = str(int(binascii.hexlify(os.urandom(16)), 16))

        # Epoch timestamp in milli seconds
        timestamp = str(int(round(time.time() * 1000)))

        data = self.API_KEY + nonce + timestamp + self.TOKEN + self.payload
        # Make sure the HMAC hash is in hex
        hmac_in_hex = hmac.new(self.API_SECRET.encode(), msg=data.encode(), digestmod=hashlib.sha256).hexdigest()

        # Authorization : base64 of hmac hash
        authorization = base64.b64encode(hmac_in_hex.encode('ascii'))

        return authorization, nonce, timestamp

    def __generate_headers(self):
        authorization, nonce, timestamp = self.__generate_hmac()

        headers = {
            'apikey': self.API_KEY,
            'token': self.TOKEN,
            'Content-type': 'application/json',
            'Authorization': authorization,
            'nonce': nonce,
            'timestamp': timestamp
        }

        return headers

    def run_transaction(self):
        headers = self.__generate_headers()
        transaction_results = requests.post(url=self.URL, data=self.payload, headers=headers)
        self.__set_transaction_response(transaction_results)

    def is_transaction_approved(self):
        """
        Returns True if transaction approved and False otherwise.

        The gateway_resp_code will always be “00” for a successful transaction, indicating that there were no errors
        attempting to process the transaction. The bank_resp_code will vary based on the response from the issuing bank.

        Complete list of bank response codes:
        https://support.payeezy.com/hc/en-us/articles/203730509-First-Data-Payeezy-Gateway-Bank-Response-Codes
        """
        # TODO find humane way to manage the codes
        VALID_HTTP_STATUS_CODES = [200, 201, 202]
        VALID_BANK_RESPONSE_CODES = ['100', '101', '102', '103', '104', '105', '106', '107', '108', '109', '110', '111',
                                     '164']

        response = self.__get_transaction_response()

        # get response status code
        http_status_code = response.status_code

        if http_status_code in VALID_HTTP_STATUS_CODES:
            # get response data
            response_data = response.json()

            # get gateway_resp_code
            gateway_response_code = response_data['gateway_resp_code']

            # compare gate_way_resp code to '00'
            gateway_response_is_valid = False
            if gateway_response_code == '00':
                gateway_response_is_valid = True

            # get bank_resp_code
            bank_response_code = response_data['bank_resp_code']

            # compare bank_resp_code to valid(successful) codes
            bank_response_is_valid = False
            if bank_response_code in VALID_BANK_RESPONSE_CODES:
                bank_response_is_valid = True

            # transaction approved if BOTH gateway_resp_code and bank_resp_code are valid(successful) codes:
            if gateway_response_is_valid and bank_response_is_valid:
                return True

        # transaction failed
        return False

    def report_transaction_error_messages(self):
        """
        Gets error messages from response to HTTP request for invalid transactions.
        Returns list.
        """
        response = self.__get_transaction_response()

        # get response data from response object
        response_data = response.json()

        # get error messages
        response_error = response_data['Error']
        response_error_messages = response_error['messages']

        # add all error messages to the report
        error_messages_to_report = []
        for response_error_message in response_error_messages:
            error_description = response_error_message['description']
            error_messages_to_report.append(error_description)

        return error_messages_to_report



import unittest
from payeezy.__make_request_bodies import *


class TestMakeRequestBodyForPurchases(unittest.TestCase):
    def test_request_body_without_reference(self):
        request_body = make_request_body_for_purchase('2499', 'visa', '4005519200000004', '1019', '123',
                                                      'Donald Duck')
        expected_request_body = {
            "merchant_ref": '',
            "transaction_type": "purchase",
            "method": "credit_card",
            "amount": '2499',
            "partial_redemption": "false",
            "currency_code": "USD",
            "credit_card": {
                "type":  'visa',
                "cardholder_name": 'Donald Duck',
                "card_number": '4005519200000004',
                "exp_date": '1019',
                "cvv": '123'
            }
        }

        self.assertEqual(request_body, expected_request_body)

    def test_request_body_with_reference(self):
        request_body = make_request_body_for_purchase('2500', 'discover', '6510000000001248', '1020', '234',
                                                      'Daisy Duck', 'Sale ref')

        expected_request_body = {
            "merchant_ref": 'Sale ref',
            "transaction_type": "purchase",
            "method": "credit_card",
            "amount": '2500',
            "partial_redemption": "false",
            "currency_code": "USD",
            "credit_card": {
                "type":  'discover',
                "cardholder_name": 'Daisy Duck',
                "card_number": '6510000000001248',
                "exp_date": '1020',
                "cvv": '234'
            }
        }

        self.assertEqual(request_body, expected_request_body)


if __name__ == '__main__':
    unittest.main()



import unittest
from payeezy import api
from payeezy import transactions


payeezy = transactions.Transaction
payeezy.API_KEY = 'y6pWAJNyJyjGv66IsVuWnklkKUPFbb0a'
payeezy.API_SECRET = '86fbae7030253af3cd15faef2a1f4b67353e41fb6799f576b5093ae52901e6f7'
payeezy.TOKEN = 'fdoa-a480ce8951daa73262734cf102641994c1e55e7cdf4c02b6'
payeezy.URL = 'https://api-cert.payeezy.com/v1/transactions'


class OverallTestCase(unittest.TestCase):

    def test_authorization(self):
        transaction = api.process_authorization('2499', 'visa', '4005519200000004', '1019', '123', 'Donald Duck')
        transaction_approved = transaction.is_transaction_approved()
        self.assertTrue(transaction_approved)


class ProcessPaymentTestCase(unittest.TestCase):

    def test_payment_with_clean_data(self):
        transaction = api.process_purchase('2499', 'visa', '4005519200000004', '1019', '123', 'Donald Duck')

        transaction_approved = transaction.is_transaction_approved()
        self.assertTrue(transaction_approved)

    def test_payment_without_transaction_total(self):
        transaction = api.process_purchase('', 'visa', '4005519200000004', '1019', '123', 'Donald Duck')

        transaction_approved = transaction.is_transaction_approved()
        self.assertFalse(transaction_approved)

        transaction_error_messages = transaction.report_transaction_error_messages()
        # according to payeezy sample, expected error message may ALSO be:
        # 'The amount provided is invalid'
        expected_error_messages = ['The amount is missing']
        self.assertEqual(transaction_error_messages, expected_error_messages)

    def test_payment_without_card_type(self):
        transaction = api.process_purchase('2499', '', '4005519200000004', '1019', '123', 'Donald Duck')

        transaction_approved = transaction.is_transaction_approved()
        self.assertFalse(transaction_approved)

        transaction_error_messages = transaction.report_transaction_error_messages()
        # according to payeezy sample, expected error message may ALSO be:
        # 'The card type cannot be empty'
        expected_error_messages = ['The card type is invalid']
        self.assertEqual(transaction_error_messages, expected_error_messages)

    def test_payment_without_card_number(self):
        transaction = api.process_purchase('2499', 'visa', '', '1019', '123', 'Donald Duck')

        transaction_approved = transaction.is_transaction_approved()
        self.assertFalse(transaction_approved)

        transaction_error_messages = transaction.report_transaction_error_messages()
        # according to payeezy sample, expected error message may ALSO be:
        # 'The card number cannot be empty',
        # 'The card number must be numeric',
        # 'The credit card number check failed'
        # 'American Express card number must be 15 digits'
        expected_error_messages = ['The card number must be 16 digits']
        self.assertEqual(transaction_error_messages, expected_error_messages)

    def test_payment_without_card_expiry(self):
        transaction = api.process_purchase('2499', 'visa', '4005519200000004', '', '123', 'Donald Duck')

        transaction_approved = transaction.is_transaction_approved()
        self.assertFalse(transaction_approved)

        transaction_error_messages = transaction.report_transaction_error_messages()
        # according to payeezy sample, expected error message SUPPOSED to be:
        # 'The card has expired'
        expected_error_messages = ['Expiry Date is invalid']
        self.assertEqual(transaction_error_messages, expected_error_messages)

    @unittest.expectedFailure
    # payeezy sandbox API does **not** check cvv and transaction is being approved
    def test_payment_without_card_cvv(self):
        transaction = api.process_purchase('2499', 'visa', '4005519200000004', '1019', '', 'Donald Duck')

        transaction_approved = transaction.is_transaction_approved()
        self.assertFalse(transaction_approved)

        transaction_error_messages = transaction.report_transaction_error_messages()
        # according to payeezy sample, expected error message SUPPOSED to be:
        # 'The cvv provided must be numeric'
        expected_error_messages = ['RELEVANT_CARD_CVV_ERROR_HERE']
        self.assertEqual(transaction_error_messages, expected_error_messages)

    @unittest.expectedFailure
    # Fails with 400 instead of the relevant API error
    def test_payment_without_cardholder_name(self):
        transaction = api.process_purchase('2499', 'visa', '4005519200000004', '1019', '123', '')

        transaction_approved = transaction.is_transaction_approved()
        self.assertFalse(transaction_approved)

        transaction_error_messages = transaction.report_transaction_error_messages()
        # according to payeezy sample, expected error message SUPPOSED to be:
        # 'The card holder name cannot be empty'
        expected_error_messages = ['RELEVANT_CARDHOLDER_NAME_ERROR_HERE']
        self.assertEqual(transaction_error_messages, expected_error_messages)


if __name__ == '__main__':
    unittest.main()





def make_request_body_for_purchase(transaction_total,
                                   card_type,
                                   card_number,
                                   card_expiry,
                                   card_cvv,
                                   cardholder_name,
                                   merchant_reference=''):

    request_body = {
        "merchant_ref": merchant_reference,
        "transaction_type": "purchase",
        "method": "credit_card",
        "amount": transaction_total,
        "partial_redemption": "false",
        "currency_code": "USD",
        "credit_card": {
            "type":  card_type,
            "cardholder_name": cardholder_name,
            "card_number": card_number,
            "exp_date": card_expiry,
            "cvv": card_cvv
        }
    }

    return request_body


def make_request_body_for_authorization(transaction_total,
                                        card_type,
                                        card_number,
                                        card_expiry,
                                        card_cvv,
                                        cardholder_name,
                                        merchant_reference=''):

    request_body = {
        "merchant_ref": merchant_reference,
        "transaction_type": "authorize",
        "method": "credit_card",
        "amount": transaction_total,
        "currency_code": "USD",
        "credit_card": {
            "type": card_type,
            "cardholder_name": cardholder_name,
            "card_number": card_number,
            "exp_date": card_expiry,
            "cvv": card_cvv
        }
    }

    return request_body



import json
from payeezy import transactions
from payeezy.__make_request_bodies import *


def process_authorization(transaction_total,
                          card_type,
                          card_number,
                          card_expiry,
                          card_cvv,
                          cardholder_name,
                          merchant_reference=''):

    request_body = make_request_body_for_authorization(transaction_total, card_type, card_number, card_expiry, card_cvv,
                                                       cardholder_name, merchant_reference)
    payload = json.dumps(request_body)

    authorization = transactions.Transaction(payload)
    authorization.run_transaction()

    return authorization


def process_purchase(transaction_total,
                     card_type,
                     card_number,
                     card_expiry,
                     card_cvv,
                     cardholder_name,
                     merchant_reference=''):

    request_body = make_request_body_for_purchase(transaction_total, card_type, card_number, card_expiry, card_cvv,
                                                  cardholder_name, merchant_reference)
    payload = json.dumps(request_body)

    purchase = transactions.Transaction(payload)
    purchase.run_transaction()

    return purchase



from setuptools import setup, find_packages


setup(
    name='payeezy',
    version='1.0.7',
    description='Unofficial Python 3 module to process basic(purchase and authorize) transactions with Payeezy',
    url='https://github.com/lemmetry/payeezy',
    author='Artem Kisel',
    author_email='artem.a.kisel@gmail.com',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Topic :: Office/Business :: Financial :: Point-Of-Sale',
    ],
    keywords='Payeezy purchase authorize transaction FirstData',
    packages=find_packages(),
    install_requires=[],
)
